#include <time.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <fstream>
#include <iostream>
#include "filterwiz/FilterFile.hh"
#include "iirutil.hh"
#include "memmap/mmap.hh"


   using namespace std;
   using namespace ligogui;

namespace filterwiz {



//////////////////////////////////////////////////////////////////////////
//                                                                      //
// Parsing utilities						        //
//                                                                      //
//////////////////////////////////////////////////////////////////////////
   typedef vector<string> tokenlist;

//______________________________________________________________________________
   const char* nextline (const char* p, const char* end, 
                     int& linelen, bool skipleadingspace = false)
   {
      bool newline = false;
      for (; p < end; ++p) {
         if ((*p == '\n') || (*p == '\r')) {
            newline = true;
         }
         else if (newline) {
            if (skipleadingspace) {
               for (; (p < end) && isspace (*p); ++p) ;
            }
            for (linelen = 0; (p + linelen < end) &&
                (p[linelen] != '\n') && (p[linelen] != '\r'); ++linelen) ;
            return p;
         }
      }
      return end;
   }

//______________________________________________________________________________
   char* putline (char* p, const char* end, const char* line)
   {
      int len = strlen (line);
      if (len >= end - p) {
         memcpy (p, line, end - p);
         return p + (end - p);
      }
      else {
         memcpy (p, line, len);
         p[len] = '\n';
         return p + len + 1;
      }
   }

//______________________________________________________________________________
   void tokens (const char* p, int max, tokenlist& tok, bool add = false)
   {
      if (!add) tok.clear();
      const char* end = p + max;
      const char* first = 0;
      for (; p < end; ++p) {
         if (isspace (*p)) {
            if (first) {
               tok.push_back (string (first, p - first));
               first = 0;
            }
         }
         else if (!first) {
            first = p;
         }
      }
      if (first) {
         tok.push_back (string (first, p - first));
      }
   }

//______________________________________________________________________________
   bool isname (const char* p) 
   {
      if (!isalpha (*p)) {
         return false;
      }
      for (; *p; ++p) {
         if (!isalnum (*p) && !(*p == '_')) {
            return false;
         }
      }
      return true;
   }

//______________________________________________________________________________
   bool isname  (const string& p) 
   {
      return isname (p.c_str());
   }

//______________________________________________________________________________
   bool isintnum  (const char* p) 
   {
      int where = 0; // 0 - start; 1 - after sign, 2 - after 1st digit
      for (; *p; ++p) {
         if (isdigit (*p)) {
            if (where <= 1) where = 2;
         }
         else if ((*p == '-') || (*p == '+')) {
            if (where == 0) where = 1;
            else 
               return false;
         }
         else {
            return false;
         }
      }
      return where == 2;
   }

//______________________________________________________________________________
   bool isintnum  (const string& p) 
   {
      return isintnum (p.c_str());
   }

//______________________________________________________________________________
   bool isnum  (const char* p) 
   {
      int where = 0; // 0 - start; 1 - after sign, 2 - after 1st digit
      // 3 - after point; 4 - after fraction didit
      // 5 - after expo, 6 - after expo sign, 7 - after expo digit
      for (; *p; ++p) {
         if (isdigit (*p)) {
            if (where <= 1) where = 2;
            else if (where == 3) where = 4;
            else if ((where == 5) || (where == 6)) where = 7;
         }
         else if ((*p == '-') || (*p == '+')) {
            if (where == 0) where = 1;
            else if (where == 5) where = 6;
            else 
               return false;
         }
         else if (*p == '.') {
            if (where == 2) where = 3;
            else 
               return false;
         }
         else if ((*p == 'e') || (*p == 'E')) {
            if ((where >= 2) && (where <= 4)) where = 5;
            else 
               return false;
         }
         else {
            return false;
         }
      }
      return (where == 2) || (where == 3) || (where == 4) || 
         (where == 7);
   }

//______________________________________________________________________________
   bool isnum  (const string& p) 
   {
      return isnum (p.c_str());
   }

//______________________________________________________________________________
   bool ismodname (const char* p) 
   {
      return isname (p);
   }

//______________________________________________________________________________
   bool ismodname  (const string& p) 
   {
      return ismodname (p.c_str());
   }



//////////////////////////////////////////////////////////////////////////
//                                                                      //
// FilterFile							        //
//                                                                      //
//////////////////////////////////////////////////////////////////////////
   FilterFile::FilterFile ()
   {
   }

//______________________________________________________________________________
   void FilterFile::clear()
   {
      fModules.clear();
   }

//______________________________________________________________________________
   void FilterFile::add (const char* name, double fsample)
   {
      for (FilterModuleList::iterator i = fModules.begin(); 
          i != fModules.end(); ++i) {
         if (strcmp (i->getName(), name) == 0) {
            *i = FilterModule (name, fsample);
            return;
         }
      }
      fModules.push_back (FilterModule (name, fsample));
   }

//______________________________________________________________________________
   void FilterFile::remove (const char* name)
   {
      for (FilterModuleList::iterator i = fModules.begin(); 
          i != fModules.end(); ++i) {
         if (strcmp (i->getName(), name) == 0) {
            fModules.erase (i);
            return;
         }
      }
   }

//______________________________________________________________________________
   FilterModule* FilterFile::find (const char* name)
   {
      for (FilterModuleList::iterator i = fModules.begin(); 
          i != fModules.end(); ++i) {
         if (strcmp (i->getName(), name) == 0) {
            return &*i;
         }
      }
      return 0;
   }

//______________________________________________________________________________
   const FilterModule* FilterFile::find (const char* name) const
   {
      for (FilterModuleList::const_iterator i = fModules.begin(); 
          i != fModules.end(); ++i) {
         if (strcmp (i->getName(), name) == 0) {
            return &*i;
         }
      }
      return 0;
   }

//______________________________________________________________________________
   bool FilterFile::valid (std::string& errmsg) const
   {
      errmsg = "";
      // loop over modules/sections
      for (FilterModuleList::const_iterator mod = fModules.begin();
          mod != fModules.end(); ++mod) {
         for (int i = 0; i < kMaxFilterSections; ++i) {
            if (!(*mod)[i].valid()) {
               if (!errmsg.empty()) errmsg += "\n";
               errmsg += mod->getName();
               char buf [256];
               sprintf (buf, "/%i", i);
               errmsg += buf; 
            }
         }
      }
      return errmsg.empty();
   }

//______________________________________________________________________________
   bool FilterFile::update()
   {
      // loop over modules/sections
      for (FilterModuleList::iterator mod = fModules.begin();
          mod != fModules.end(); ++mod) {
         for (int i = 0; i < kMaxFilterSections; ++i) {
            if (!(*mod)[i].update()) {
               return false; 
            }
         }
      }
      return true;
   }

//______________________________________________________________________________
   bool FilterFile::read (const char* filename)
   {
      if (!filename) {
         cerr << "Illegale file name" << endl;
         return false;
      }
      string oldfilename = fFilename;
      fFilename = filename;
      gdsbase::mmap mfile (filename, std::ios_base::in);
      if (!mfile) {
         cerr << "Unable to open file " << filename << endl;
         fFilename = oldfilename;
         return false;
      }
      bool succ = read ((const char*)mfile.get(), mfile.size()) == 
         (int)mfile.size();
      if (!succ) {
         fFilename = oldfilename;
      }
      return succ;
   }

//______________________________________________________________________________
   int FilterFile::read (const char* beg, int maxlen)
   {
      // clear old modules
      fModules.clear();
      // check magic string
      if ((maxlen < (int)strlen (kMagic)) ||
         (strncmp (beg, kMagic, strlen (kMagic)) != 0)) {
         clear();
         cerr << "Not an online filter file" << endl;
         return 0;
      }
      // first look for filter module names
      const char* end = beg + maxlen;
      int linelen;
      for (const char* p = beg; p < end; p = nextline (p, end, linelen)) {
         if ((linelen > (int)strlen (kModuleDef)) &&
            (strncmp (p, kModuleDef, strlen (kModuleDef)) == 0)) {
            tokenlist tok;
            p += strlen (kModuleDef);
            linelen -= strlen (kModuleDef);
            tokens (p, linelen, tok);
            for (tokenlist::iterator i = tok.begin(); 
                i != tok.end(); ++i) {
               if (ismodname (i->c_str())) {
                  add (i->c_str(), 1E-12);
               }
               else {
                  cerr << "Illegale file module name " << *i << endl;
               }
            }
         }
      }
   
      // next look for filter module sampling rate
      for (const char* p = beg; p < end; p = nextline (p, end, linelen)) {
         if ((linelen > (int)strlen (kModuleSamplingDef)) &&
            (strncmp (p, kModuleSamplingDef, 
                     strlen (kModuleSamplingDef)) == 0)) {
            tokenlist tok;
            tokens (p + strlen (kModuleSamplingDef), 
                   linelen - strlen (kModuleSamplingDef), tok);
            if ((tok.size() == 2) && ismodname (tok[0]) && isnum (tok[1])) {
               FilterModule* mod = find (tok[0].c_str());
               if (mod) {
                  mod->setFSample (atof (tok[1].c_str()));
               }
               else {
                  cerr << "Sampling rate for unknown filter module " 
                     << tok[0] << endl;
               }
            }
            else {
               cerr << "Illegale smapling rate specification:" << endl 
                  << "  " << string (p, linelen) << endl;
            }
         }
      }
   
      // if we got no filter modules, start a scan
      if (fModules.empty()) {
         for (const char* p = beg; p < end; p = nextline (p, end, linelen)) {
            if ((*p == '#') || isspace (*p)) {
               continue;
            }
            tokenlist tok;
            tokens (p, linelen, tok);
            if ((tok.size() > 0) && ismodname (tok[0].c_str())
               && !find (tok[0].c_str())) {
               cout << "Warning: Filter module " << tok[0] << 
                  " added." << endl;
               add (tok[0].c_str(), 1E-12);
            }
         }
      }
   
      // need at least one module
      if (fModules.empty()) {
         clear();
         cerr << "No filter module specification" << endl;
         return 0;
      }
   
      // guess sample rate if necessary
      for (FilterModuleList::iterator i = modules().begin(); 
          i != modules().end(); ++i) {
         if (i->getFSample() <= 1E-10) {
            i->guessFSample (fFilename.c_str());
            cout << "Warning: Guessing sampling rate of " << i->getName() <<
               " to be " << i->getFSample() << endl;
         }
      }
   
      // now read modules/sections
      string header;
      for (const char* p = nextline (beg, end, linelen); 
          p < end; p = nextline (p, end, linelen)) {
         // skip filter info lines
         if (((linelen > (int)strlen (kModuleDef)) && 
             (strncmp (p, kModuleDef, strlen (kModuleDef)) == 0)) ||
            ((linelen > (int)strlen (kModuleSamplingDef)) &&
            (strncmp (p, kModuleSamplingDef, 
                     strlen (kModuleSamplingDef)) == 0)) ||
            ((linelen > (int)strlen (kSectionDesignDef)) &&
            (strncmp (p, kSectionDesignDef, 
                     strlen (kSectionDesignDef)) == 0))) {
            continue;
         }
         // add comments to header
         if (*p == '#') {
            if (header.empty()) header += '\n';
            header += string (p, linelen);
            continue;
         }
         // found something
         tokenlist tok;
         tokens (p, linelen, tok);
         if (tok.empty()) {
            header += '\n';
            continue;
         }
         // check if correctly formated section spec
         if ((tok.size() != 12) || !ismodname (tok[0]) || 
            !isintnum (tok[1]) || !isintnum (tok[2]) ||
            !isintnum (tok[3]) || !isnum (tok[4]) ||
            !isnum (tok[5]) || !isnum (tok[7]) ||
            !isnum (tok[8]) || !isnum (tok[9]) ||
            !isnum (tok[10]) || !isnum (tok[11])) {
            cerr << "Illegale filter section:" << endl <<
               "  " << string (p, linelen) << endl;
            continue;
         }
         FilterModule* mod = find (tok[0].c_str());
         if (!mod) {
            cerr << "Unknown filter module " << tok[0] << endl;
            continue;
         }
         int index = atoi (tok[1].c_str());
         if ((index < 0) || (index >= kMaxFilterSections)) {
            cerr << "Illegale section number " << tok[1] << endl;
            continue;
         }
         input_switching  inpsw =  
            (input_switching)(atoi (tok[2].c_str()) / 10);
         if ((inpsw != kAlwaysOn) && (inpsw != kZeroHistory)) {
            cerr << "Illegale input switch " << tok[2] << endl;
            continue;
         }
         output_switching outsw = 
            (output_switching)(atoi (tok[2].c_str()) % 10);
         if ((outsw != kImmediately) && (outsw != kRamp) &&
            (outsw != kInputCrossing) && (outsw != kZeroCrossing)) {
            cerr << "Illegale output switch " << tok[2] << endl;
            continue;
         }
         int sosnum = atoi (tok[3].c_str());
         if ((sosnum < 1) || (sosnum > kMaxFilterSOS)) {
            cerr << "Illegale SOS number " << tok[3] << endl;
            continue;
         }
         double ramp = 0;
         double tolerance = 0;
         double timeout = 0;
         switch (outsw) {
            case kImmediately: 
               break;
            case kRamp:
               ramp = atof (tok[4].c_str()) / mod->getFSample();
               if (ramp <= 0) {
                  cerr << "Illegale ramp time " << tok[4] << endl;
                  continue;
               }
               break;
            case kInputCrossing:
            case kZeroCrossing:
               tolerance = atof (tok[4].c_str());
               timeout = atof (tok[5].c_str()) / mod->getFSample();
               if (tolerance < 0) {
                  cerr << "Illegale switch tolerance " << tok[4] << endl;
                  continue;
               }
               if (timeout < 0) {
                  cerr << "Illegale timeout " << tok[5] << endl;
                  continue;
               }
               break;
         }
         // OK, we got the 1st line: Now check for additional SOS lines
         for (int i = 1; i < sosnum; ++i) {
            p = nextline (p, end, linelen);
            if (p >= end) {
               cerr << "Not enough lines in file" << endl;
               break;
            }
            tokens (p, linelen, tok, true);
         }
         if ((int)tok.size() != 8 + 4 * sosnum) {
            cerr << "Not enough second order sections for " 
               << tok[0] << endl;
            continue;
         }
         bool ok = true;
         for (int i = 12; i < 8 + 4 * sosnum; ++i) {
            if (!isnum (tok[i])) ok = false;
         }
         if (!ok) {
            cerr << "Filter coefficients for " << tok[0] 
               << " are not all numbers" << endl;
            continue;
         }
         // Finally we have a valid filter section!
         FilterSection& sect = mod->operator[] (index);
         sect.filter().reset();
         sect.filter().gain (atof (tok[7].c_str()));
         for (int i = 8; i < 8 + 4 * sosnum; i += 4) {
            sect.filter().biquad (1., atof (tok[i+2].c_str()),
                                 atof (tok[i+3].c_str()),
                                 atof (tok[i].c_str()),
                                 atof (tok[i+1].c_str()));
         }
         sect.setName (tok[6].c_str());
         sect.setInputSwitch (inpsw);
         sect.setOutputSwitch (outsw);
         sect.setRamp (ramp);
         sect.setTolerance (tolerance);
         sect.setTimeout (timeout);
         sect.setHeader (header.c_str());
         header = "";
      }
   
      // last look for filter section design strings
      for (const char* p = beg; p < end; p = nextline (p, end, linelen)) {
         // look for design string
         if ((linelen > (int)strlen (kSectionDesignDef)) &&
            (strncmp (p, kSectionDesignDef, 
                     strlen (kSectionDesignDef)) == 0)) {
            // tokenize string
            tokenlist tok;
            tokens (p + strlen (kSectionDesignDef), 
                   linelen - strlen (kSectionDesignDef), tok);
            if ((tok.size() >= 3) && ismodname (tok[0]) &&
               isintnum (tok[1])) {
               // found a valid design
               FilterModule* mod = find (tok[0].c_str());
               int index = atoi (tok[1].c_str());
               if (mod && (index >= 0) && (index < kMaxFilterSections)) {
                  // found a valid module
                  string s (p + strlen (kSectionDesignDef),
                           linelen - strlen (kSectionDesignDef));
                  int indent = strlen (kSectionDesignDef);
                  // delete module name
                  string::size_type pos = s.find (tok[0].c_str());
                  if (pos != string::npos) {
                     s.erase (0, pos + tok[0].size());
                     indent += pos + tok[0].size();
                  }
                  // delete section number
                  pos = s.find (tok[1].c_str());
                  if (pos != string::npos) {
                     s.erase (0, pos + tok[1].size());
                     indent += pos + tok[1].size();
                  }
                  // delete leading white space
                  while (!s.empty() && isspace (s[0])) {
                     s.erase (0, 1);
                     ++indent;
                  }
                  // delete trailing white space
                  while (!s.empty() && isspace (s[s.size()-1])) 
                     s.erase (s.size()-1, 1);
                  // look for line continuation
                  while (!s.empty() && (s[s.size()-1] == '\\') && 
                        (p < end)) {
                     // remove continuation characters
                     do {
                        s.erase (s.size()-1, 1);
                     } while (!s.empty() && (isspace (s[s.size()-1])));
                     // add new line
                     s += "\n";
                     // look for new line
                     p = nextline (p, end, linelen);
                     string n (p, linelen);
                     // delete leading white space and #
                     for (int i = 0; (i < indent) && !n.empty() && 
                         (isspace (n[0]) || (n[0] == '#')); ++i) {
                        n.erase (0, 1);
                     }
                     // delete trailing white space
                     while (!n.empty() && isspace (n[n.size()-1])) 
                        n.erase (n.size()-1, 1);
                     s += n;
                  }
                  // set design string
                  (*mod)[index].setDesign (s.c_str());
               }
               else if (!mod) {
                  cerr << "Design for unknown filter module " 
                     << tok[0] << endl;
               }
               else {
                  cerr << "Illegale section number " << tok[0] <<
                     "/" << tok[1] << endl;
               }
            }
            else {
               cerr << "Illegale design specification:" << endl 
                  << "  " << string (p, linelen) << endl;
            }
         }
      }
   
      // now check filter if design strings are ok
      for (FilterModuleList::iterator i = fModules.begin();
          i != fModules.end(); ++i) {
         i->checkDesign();
      }
   
      return maxlen;
   }

//______________________________________________________________________________
   bool FilterFile::write (const char* filename)
   {
      if (!filename) {
         cerr << "Illegale file name" << endl;
         return false;
      }
      if (!update()) {
         cerr << "Invalid filter design" << endl;
         return false;
      }
      ofstream out (filename);
      if (!out) {
         cerr << "Unable to open file " << filename << endl;
         return false;
      }
      int size = 64*1024;
      char* p  = 0;
      int len;
      do {
         if (p) delete [] p;
         size *= 2;
         p = new char [size];
         len = write (p, size);
      } while (len >= size);
      out.write (p, len);
      delete [] p;
      if (!out) {
         cerr << "Unable to write file " << filename << endl;
      }
      return (out);
   }

//______________________________________________________________________________
   int FilterFile::write (char* beg, int maxlen)
   {
      if (!update()) {
         cerr << "Invalid filter design" << endl;
         return false;
      }
   
      char* p = beg;
      char* end = beg + maxlen;
      // write magic header
      p = putline (p, end, kMagic);
      // write warning
      p = putline (p, end, 
                  "#\n# Computer generated file: DO NOT EDIT\n#");
      // write filter module names
      string line = kModuleDef;
      for (FilterModuleList::const_iterator mod = fModules.begin();
          mod != fModules.end(); ++mod) {
         line += string (" ") + mod->getName();
         if (line.size() > 70) {
            p = putline (p, end, line.c_str());
            line = kModuleDef;
         }
      }
      if (line.size() > strlen (kModuleDef)) {
         p = putline (p, end, line.c_str());
      }
      p = putline (p, end, "#");
      // write filter sections
      for (FilterModuleList::const_iterator mod = fModules.begin();
          mod != fModules.end(); ++mod) {
      
         // write title
         char buf[256];
         p = putline (p, end, "##########################################"
                     "######################################");
         sprintf (buf, "### %-72s ###", mod->getName());
         p = putline (p, end, buf);
         p = putline (p, end, "##########################################"
                     "######################################");
      
         // write sampling rate
         sprintf (buf, "%s %s %g", kModuleSamplingDef,
                 mod->getName(), mod->getFSample());
         p = putline (p, end, buf);
      
         // write design string
         for (int i = 0; i < kMaxFilterSections; ++i) {
            const FilterSection& sect = mod->operator[] (i);
            if (!sect.empty()) {
               string s;
               char buf[256];
               sprintf (buf, "%s   %s %i ", kSectionDesignDef,
                       mod->getName(), i);
               s += buf;
               string linebreak = " \\## ";
               linebreak.append (s.size() - 2, ' ');
               string cmd = sect.getDesign();
               string::size_type pos;
               while ((pos = cmd.find ('\n')) != string::npos) {
                  cmd.erase (pos, 1);
                  cmd.insert (pos, linebreak.c_str());
               }
               while ((pos = cmd.find ("##")) != string::npos) {
                  cmd[pos] = '\n';
               }
               s += cmd;
               p = putline (p, end, s.c_str());
            }
         }
         sprintf (buf, "### %-72s ###", "");
         p = putline (p, end, buf);
      
         // write coefficients
         for (int i = 0; i < kMaxFilterSections; ++i) {
            const FilterSection& sect = mod->operator[] (i);
            if (!sect.empty()) {
               int order = iirorder (sect.filter().get());
               if (order < 0) {
                  cerr << "Not an IIR filter" << endl;
                  continue;
               }
               int nba;
               double* coeff = new double [1 + 4*order];
               if (!iir2z (sect.filter().get(), nba, coeff, "o")) {
                  cerr << "Unable to obtain online filter coefficients" 
                     << endl;
                  continue;
               }
               //p = putline (p, end, sect.getHeader());
               char buf[1024];
               int sosnum = (nba - 1) / 4;
               if ((sosnum < 1) || (sosnum > kMaxFilterSOS)) {
                  cerr << "Invalid number of SOSs " << sosnum << endl;
                  continue;   
               }
               int arg1 = 0;
               int arg2 = 0;
               int sw = 10*(int)sect.getInputSwitch() +
                  (int)sect.getOutputSwitch();
               switch (sect.getOutputSwitch()) {
                  case kImmediately: 
                     break;
                  case kRamp:
                     arg1 = (int)(sect.getRamp() * mod->getFSample() + 0.5);
                     break;
                  case kInputCrossing:
                  case kZeroCrossing:
                     arg1 = (int)(sect.getTolerance() + 0.5);
                     arg2 = (int)(sect.getTimeout() * mod->getFSample()  + 0.5);
                     break;
               }
               string sect_name = validSectionName (sect.getName());
               sprintf (buf, "%-8s %i %2i %i %6i %6i %-10s %22.14g "
                       "%20.14f %20.14f %20.14f %20.14f", 
                       mod->getName(), i, sw, sosnum, arg1, arg2, 
                       sect_name.c_str(), coeff[0], coeff[1], coeff[2],
                       coeff[3], coeff[4]);
               p = putline (p, end, buf);
               for (int j = 1; j < sosnum; ++j) {
                  sprintf (buf, "%63s %20.14f %20.14f %20.14f %20.14f",
                          "", coeff[1+4*j], coeff[2+4*j], 
                          coeff[3+4*j], coeff[4+4*j]);
                  p = putline (p, end, buf);
               }
            }
         }
      
         // empty line
         p = putline (p, end, "");
      }
   
      return p - beg;
   }

//______________________________________________________________________________
   std::string FilterFile::validSectionName (const char* name)
   {
      std::string s;
      for (const char* p = name; *p != 0; ++p) {
         if (isgraph (*p)) {
            s += *p;
         }
      }
      if (s.empty()) s = "Unknown";
      return s;
   }

}
