#include <detect.h>

/*----------------------------------------------------------------------*
 *	Function: VOID Ibingo() - Sets parameters 			*
 *----------------------------------------------------------------------*
 *	Arguments:	No arguments					*
 *----------------------------------------------------------------------*
 *	Returns:	Nothing (VOID)					*
 *----------------------------------------------------------------------*
 *	Fatal Errors:	This routine does not have fatal exits		*
 *----------------------------------------------------------------------*
 *	Ibingo is called when an event has been detected.  (Here it is	*
 *	 useful to note that although Ibingo is called when an event	*
 * 	 is detected, an event is not declared, ie routine Event re-	*
 *	 turns true, until enough P-T values have been processed to 	*
 *	 estimate the period of the signal.)  The purpose of Ibingo 	*
 *	 is to set parameters for processing the interval of the event.	*
 *	 The interval of the event is interval in which E_detect re-	*
 *	 turns TRUE.  Typically the interval of the event is 		*
 *	 itc x wait_blk samples, where usually itc is 4 and wait_blk	*
 *	 is 1014 (samples).						*
 ************************************************************************/

VOID Ibingo()
{
	con_ptr->epf = FALSE; 
	con_ptr->evon = TRUE; 
	con_ptr->icheck = FALSE; 
	con_ptr->itc = NOR_OUT;
	con_ptr->nn = 0;	/*  Initialize for Count_dn */
	return;
}
#ifdef JNMCOMMENT
/************************************************************************

			EXPLANATION OF VARIABLES
	
	epf	-- event possible flag, set to TRUE by Event when
		    a signal is deemed possibly to be in progress (but
		    not yet detected), set to FALSE upon detection.

	evon	-- event on flag, set to TRUE when an event is detected,
		    set to FALSE when NOR_OUT-EV_OFF x wait_blk samples
		    have been processed by Count_dn.  Typically EV_OFF = 2,
		    wait_block = 1014.

	icheck	-- used in routine Event to inhibit declaring an event
		    until enough P-T values have been processed to 
		    estimate the period of the signal. Set to TRUE
		    in Event when these values have been processed.

	itc	-- counter for the interval of the event.  Decremented
		    to zero when the last sample of the interval of
		    the event has been processed.  The interval of
		    the event typically is itc x wait_blk samples.  
		    (The interval will be longer if a retrigger occurs.)
		    Typically, itc = 4, wait_blk = 1014.  See Count_dn.

	nn	-- an index used in routine Count_dn.

	NOR_OUT --  typically 4.  It is a
		    factor of wait_blk through itc.

	th1	-- threshold for detection of event. See  Xth, Event. (Counts.)
	th2	-- threshold for detection of event. See  Xth, Event. (Counts.)
	th3	-- threshold for estimating onset of event. See  Xth, Onsetq.
		    (Counts.)
	thx	-- upper bound for estimate of twosd. See P_one, Xth. (Counts.)

 *****************************************************************************/

#endif
