#ifndef STRAIN_WATCH_HH
#define STRAIN_WATCH_HH

#include "DatEnv.hh"
#include "MonServer.hh"
#include "OperStateCondList.hh"
#include "PSD.hh"
#include "Hanning.hh"
#include "FSpectrum.hh"

#include "StrainGet.hh"

using namespace std;



class StrainWatch : public DatEnv, MonServer
{
	public:
		StrainWatch( int argc, const char *argv[] );
		~StrainWatch();
		
		void ProcessData();
		void Attention();
		
		void setDefaultParams();
		void readCommandLine( int argc, const char *argv[] );
		string loadParams( istream *s );
		void readConfigFile();
		void updateParams();
		
		void dumpUsage();
	private:
		int verbosity;
		string configFile;
		int timeStride;
		int downsample;
		long segmentStartTime;
		int wings;
		double factorsTime;
		string ifo;
		bool debugMode;
		string lockConditions;
		string oscFile;
		int intervals;

		string asqChannel;
		string excChannel;
		string darmChannel;
		string darmErrChannel;

		long lastFillTime;
		int numSecsGoodData;
		bool isFirstRun;
		OperStateCondList mOSC;
		
		double *asqData;
		double *darmData;
		double *darmErrData;
		double *excData;
		
		PSD *psd;
		PSD *psd1min;
		Hanning *window;

		double *alphaRe, *alphaIm, *betaRe, *betaIm, *alphabetaRe, *alphabetaIm;
		TSeries *alphaReTS, *alphaImTS, *betaReTS, *betaImTS, *alphabetaReTS, 
			*alphabetaImTS;
		
		FSpectrum arFS, aiFS, brFS, biFS, abrFS, abiFS;
		FSpectrum arFSave, aiFSave, brFSave, biFSave, abrFSave, abiFSave;

		FSpectrum hFS, hRFS, hCFS;
		FSpectrum hFSave, hRFSave, hCFSave;
		bool aveDataSet;
		
		StrainGetArguments strGetParams;
		StrainGet *strGet;
};

#endif
