/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "TrigClient.hh"
#include "DQSegWriter.hh"
#include "LdasDBWriter.hh"
#ifndef DMTOFFLINE
#include "MsgWriter.hh"
#endif
#include "lmsg/ErrorList.hh"
#include <iostream>

using namespace trig;
using namespace std;

extern trig::TrigProc* ProcIdent;

trig::TrigProc* ProcIdent = 0;

//======================================  Construct and register the client.
TrigClient::TrigClient(void) 
  : mDebug(0), mWriter(0)
{
    enroll(TrigWriter::kNone);
}

//======================================  Construct and register the client.
TrigClient::TrigClient(TrigWriter::trig_mode mode) 
  : mDebug(0), mWriter(0)
{
    if (mode != TrigWriter::kNone) enroll(mode);
}

//======================================  Unregister and go away
TrigClient::~TrigClient(void) {

    //----------------------------------  Trigger manager interface
    if  (mWriter) {
	mWriter->write(mTableFile.c_str());
	delete mWriter;
	mWriter=0;
    }
}

//======================================  Register with the trigger manager.
lmsg::error_type 
TrigClient::enroll(TrigWriter::trig_mode mode) {
    const char* trigf = getenv("TRIGGER_FILE");
    const char* segf  = getenv("SEGMENT_FILE");
    lmsg::error_type rc = lmsg::OK;

    //----------------------------------  Handle ldas trigger writer
    if (mode == TrigWriter::kWriter || (!mWriter && trigf)) {
	if (trigf) mTableFile = trigf;
	if (!mWriter) mWriter = new LdasDBWriter;
	if (mDebug) cout << "Trigger output to file: " << mTableFile << endl;

    //----------------------------------  Handle segment writer
    } else if (mode == TrigWriter::kSegWrt || (!mWriter && segf)) {
	if (segf) mTableFile = segf;
	if (!mWriter) mWriter = new DQSegWriter;
	if (mDebug) cout << "Segment output to file: " << mTableFile << endl;

#ifndef DMTOFFLINE
    //----------------------------------  look up the trigger manager.
    } else if (!mWriter) {
	mWriter = new MsgWriter;
	if (mDebug) cout << "Output to Trigger Manager"<< endl;
#endif
    }
    if (mWriter) {
        mWriter->setDebug(mDebug);
	rc = mWriter->setProcess(*ProcIdent);
    }
    return rc;
}

//======================================  Log a trigger
lmsg::error_type
TrigClient::sendSegment(const Segment& seg) {
    lmsg::error_type rc = lmsg::OK;
    if (!mWriter) rc = enroll(TrigWriter::kNone);
    if (mWriter)  rc = mWriter->addSegment(seg);
    return rc;
}

//======================================  Log a trigger
lmsg::error_type
TrigClient::sendTrigger(const TrigBase& trigger) {
    lmsg::error_type rc = lmsg::OK;
    if (!mWriter) rc = enroll(TrigWriter::kNone);
    if (mWriter)  rc = mWriter->addTrigger(trigger);
    return rc;
}

//======================================  Set the output file name
void
TrigClient::setTableFile(const char* name) {
    mTableFile = name;
}

//======================================  Set the debug level.
void 
TrigClient::setDebug(lmsg::index_type level) {
    mDebug = level;
    if (mWriter) mWriter->setDebug(level);
}
