dnl -*- m4 -*-
dnl----------------------------------------------------------------------
dnl This is a common set of macros to be used in the configuration of
dnl	the GDS software
dnl----------------------------------------------------------------------

dnl ---------------------------------------------------------------------
 
AC_DEFUN([AC_GDS_PROG_CC],
[ AC_PROG_CC
if test "${GCC}" = "yes"
then
  touch ,t.c
  ac_dmt_default_cc_includes="`${CC} -v ,t.c 2>&1 | \
        sed -n -e '/\#include/,/End of search list\./p' | \
        egrep '^ '`"
  rm ,t.c
fi
GDS_CC_FLAVOR="generic"
case $CC in
gcc|*/gcc) GDS_CC_FLAVOR="gnu";;
cc|*/cc)
    case "`$CC -V 2>&1`" in
    *Sun\ C\ *)
      GDS_CC_FLAVOR="SunPRO"
      ;;
    esac
    ;;
esac
AM_CONDITIONAL([GDS_CC_SUNPRO],[test x$GDS_CC_FLAVOR = xSunPRO])
])
 
dnl ---------------------------------------------------------------------
 
AC_DEFUN([AC_GDS_PROG_CXX],
[ AC_PROG_CXX
if test "${GXX}" = "yes"
then
  touch ,t.cc
  ac_dmt_default_cxx_includes="`${CXX} -v ,t.cc 2>&1 | \
        sed -n -e '/\#include/,/End of search list\./p' | \
        egrep '^ '`"
  rm ,t.cc
fi
GDS_CXX_FLAVOR="generic"
case $CXX in
gcc|*/gcc|g++|*/g++) GDS_CXX_FLAVOR="gnu";;
CC|*/CC)
    case "`$CXX -V 2>&1`" in
    *Sun\ C++*)
      GDS_CXX_FLAVOR="SunPRO"
      ;;
    esac
    ;;
esac
AC_SYS_LARGEFILE
AM_CONDITIONAL([GDS_CXX_SUNPRO],[test x$GDS_CXX_FLAVOR = xSunPRO])
])
 

dnl======================================================================
dnl GDS_CHECK_ROOT
dnl   Check if ROOT is installed
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_ROOT],
[
rootcflags=
rootlibs=
rootglibs=
echo $ECHO_N "checking for ROOT ... $ECHO_C"
if test "x$rootdir" = "x"
then
  rootdir=$ROOTSYS
fi
if test -x $rootdir/bin/root-config
then
  rootbin=$rootdir/bin
  rootcflags=`$rootbin/root-config --cflags`
  rootlibs=`$rootbin/root-config --libs`
  rootglibs=`$rootbin/root-config --glibs`
  rootlib=`echo $rootlibs | sed -e 's|^-L\([[^ ]]*\).*$|\1|'`
  rootinc=`echo $rootcflags | sed -e 's|^-I\([[^ ]]*\).*$|\1|'`
  rootcint="$rootbin/rootcint"
  rootbaselibs="-L`$rootbin/root-config --libdir` -lCint -lCore `$rootbin/root-config --auxlibs`"
  LD_LIBRARY_PATH="$rootlib:$LD_LIBRARY_PATH"
else
  echo "no $ECHO_C"
  echo "ROOT must be on the system - Aborting"
  exit 1
fi
echo "$rootdir"
AC_SUBST(rootdir)
AC_SUBST(rootbin)
AC_SUBST(rootlib)
AC_SUBST(rootinc)
AC_SUBST(rootcflags)
AC_SUBST(rootlibs)
AC_SUBST(rootglibs)
AC_SUBST(rootbaselibs)
AC_SUBST(rootcint)
AC_SUBST(LD_LIBRARY_PATH)
])

dnl======================================================================
dnl GDS_CHECK_FFTW3
dnl   Check if FFTW3 is installed
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_FFTW3],
[
AC_CHECK_LIB(fftw3f, fftwf_execute_dft, have_fftw3f="yes", have_fftw3f="no")
AC_CHECK_LIB(fftw3,  fftw_execute_dft,  have_fftw3d="yes", have_fftw3d="no")
AM_CONDITIONAL([require_fftw3], [test "$have_fftw3f" = "no" -a "$have_fftw3d" = "no" ])
if test "$have_fftw3f" = "yes"; then
  pkgf=/usr
  pkgl=$pkgf/lib
  for d in `echo "$LD_LIBRARY_PATH" | sed -e "s/:/ /g"` ; do
    if test -f "$d/libfftw3f.so" -o -f "$d/libfftw3f.a" ; then
      pkgl=`echo $d | sed -e 's,/$,,g'`
      pkgf=`echo $pkgl | sed -e 's,/lib\(64\)*$,,'`
      echo $pkgf
      break
    fi
  done
  fftwcflags="-I$pkgf/include -DFFTW3"
  fftwlibs="-L$pkgl -lfftw3f"
  if test "$have_fftw3d" = "yes"; then
    fftwcflags="$fftwcflags -DFFT_DOUBLE"
    fftwlibs="$fftwlibs -lfftw3"
  fi
else
  fftwcflags=""
  fftwlibs=""
fi
AC_SUBST(fftwcflags)
AC_SUBST(fftwlibs)
])

dnl======================================================================
dnl GDS_CHECK_ZLIB
dnl   Check if ZLIB is installed
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_ZLIB],
[
found_zutil=no
zlibcflags=
zliblibs="-lz"
have_libz=no
AC_CHECK_LIB([z], inflate, have_libz="yes", have_libz="no")
libp=`echo $LD_LIBRARY_PATH | sed -e s,::,:,g -e 's,/lib\(64\)*\:, ,g' -e 's,/lib\(64\)*$,,g'`
for p in $libp /usr/local /usr ; do
  if test -f $p/include/zutil.h ; then
    zlibcflags="-I$p/include"
    zliblibs="-L$p/lib -lz"
    found_zutil=yes
    break
  elif test -f $p/include/zlib.h ; then
    zlibcflags="-I$p/include"
  fi
done
AM_CONDITIONAL([have_zutil], [test "$found_zutil" = "yes"])
AC_SUBST(zlibcflags)
AC_SUBST(zliblibs)
if test $have_libz = "no" ; then
  AC_MSG_ERROR([libz not found])
fi
])

dnl======================================================================
dnl GDS_CHECK_FRAMECPP
dnl   Check if framecpp is installed
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_FRAMECPP],
[
have_framecpp=
PKG_CHECK_MODULES(FRAMECPP, framecpp, have_framecpp="yes", have_framecpp="no")
if test "${have_framecpp}" = "yes"
then
framecpplibs="$FRAMECPP_LIBS"
framecppcflags="$FRAMECPP_CFLAGS"
fi
if test "${have_framecpp}" = "no"
then
framecppcflags="-I../../External/include"
framecpplibs="-L../../External/lib -lframecpp -lframecppcmn -lframecpp6 -lframecpp3_4_5 -lgeneral"
fi
AM_CONDITIONAL([require_framecpp], [test "$have_framecpp" = no])
AC_SUBST(framecpplibs)
AC_SUBST(framecppcflags)
])


dnl======================================================================
dnl GDS_CHECK_LAL
dnl   Check if lal is installed
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_LAL],
[
lal_pkgok=
PKG_CHECK_MODULES(LAL, lal, lal_pkgok="yes", lal_pkgok="no")
if test "${lal_pkgok}" = "yes"
then
lallibs="$LAL_LIBS"
lalcflags="$LAL_CFLAGS"
else
lallibs=
lalcflags=
fi
AM_CONDITIONAL([have_lal], [test "$lal_pkgok" = yes])
AC_SUBST(lallibs)
AC_SUBST(lalcflags)
])


dnl======================================================================
dnl GDS_CHECK_FR
dnl   Check if FR is installed
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_FR],
[
have_fr=
PKG_CHECK_MODULES(FRAME, libframe, have_fr="yes", have_fr="no")
if test "${have_fr}" = "yes"
then
frlibs="$FRAME_LIBS"
frcflags="$FRAME_CFLAGS"
fi
if test "${have_fr}" = "no"
then
frlibs="-lFrame"
frcflags=""
fi
AM_CONDITIONAL([require_fr], [test "$have_fr" = no])
AC_SUBST(frlibs)
AC_SUBST(frcflags)
])

dnl======================================================================
dnl GDS_CHECK_METAIO
dnl   Check if METAIO is installed
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_METAIO],
[
have_metaio=
PKG_CHECK_MODULES(METAIO, libmetaio, have_metaio="yes", have_metaio="no")
if test "${have_metaio}" = "yes"
then
metaiolibs="$METAIO_LIBS"
metaiocflags="$METAIO_CFLAGS"
fi
if test "${have_metaio}" = "no"
then
metaiolibs="-lmetaio"
metaiocflags=""
fi
AM_CONDITIONAL([require_metaio], [test "$have_metaio" = no])
AC_SUBST(metaiolibs)
AC_SUBST(metaiocflags)
])

dnl======================================================================
dnl GDS_BUILD_TYPE
dnl   Check build type: online, offline, ...
dnl======================================================================

AC_DEFUN([AC_GDS_BUILD_TYPE],
[
echo $ECHO_N "checking for GDSBUILD ... $ECHO_C"
if test "x$buildtype" = "x"
then
  buildtype=$GDSBUILD
fi
case "x$buildtype" in
x)
  echo "not specified default: offline $ECHO_C"
  buildtype=offline
  ;;
xonline)
  echo "$buildtype"
  ;;
xoffline)
  echo "$buildtype"
  ;;
*)
  echo "$buildtype $ECHO_C"
  echo "GDSBUILD must be specified as online or offline - Aborting"
  exit 1
  ;;
esac

AM_CONDITIONAL([online], [test "$buildtype" = "online"])
AM_CONDITIONAL([offline], [test "$buildtype" = "offline"])
])

dnl======================================================================
dnl GDS_CHECK_POSIX
dnl	
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_POSIX],
[
posixflags="-D_POSIX_C_SOURCE=199506L"
unix98flags="-D_XOPEN_SOURCE=500"
case "${host}" in
*linux*) unix98flags="-D_XOPEN_SOURCE=600" ;;
esac
])

dnl======================================================================
dnl AC_GDS_FINISH - Things to be done at the very end for GDS
dnl======================================================================
 
AC_DEFUN([AC_GDS_FINISH],
[
  dnl ===================================================================
  dnl  Setup $architecture
  dnl ===================================================================
architecture=
case "${host}" in
*linux*) architecture="linux" ;;
*solaris*) architecture="solaris" ;;
esac
AC_SUBST(architecture)
  dnl ===================================================================
  dnl  Setup $syslibs
  dnl ===================================================================
syslibs=
case "${host}" in
*linux*) syslibs= ;;
*solaris*) syslibs="-lrt" ;;
esac
AC_SUBST(syslibs)
  dnl ===================================================================
  dnl  Setup $sysmtlibs
  dnl ===================================================================
sysmtlibs=
case "${host}" in
*linux*) sysmtlibs="-lpthread" ;;
*solaris*) sysmtlibs="-lrt -lpthread" ;;
esac
AC_SUBST(sysmtlibs)
  dnl ===================================================================
  dnl  Setup $netlib
  dnl ===================================================================
netlib=
case "${host}" in
*linux*) netlib="" ;;
*solaris*) netlib="-lnsl -lsocket" ;;
esac
AC_SUBST(netlib)
  dnl ===================================================================
  dnl  Setup $readlinelibs
  dnl ===================================================================
readlinelibs=
case "${host}" in
*linux*) readlinelibs="-lreadline -ltermcap" ;;
*solaris*) readlinelibs="-lreadline -ltermcap" ;;
esac
AC_SUBST(readlinelibs)
  dnl ===================================================================
  dnl  Specify warning message level
  dnl ===================================================================
case "$GDS_CC_FLAVOR" in
gnu)
  CFLAGS="-pipe -fPIC -Wall $posixflags $unix98flags"
  ;;
SunPRO)
  CFLAGS="-v -KPIC -xc99=none"
  ;;
esac
case "$GDS_CXX_FLAVOR" in
gnu)
  CXXFLAGS="-pipe -fPIC -Wall -Woverloaded-virtual $posixflags $unix98flags"
  ;;
SunPRO)
  CXXFLAGS="-KPIC"
  ;;
esac
  dnl ===================================================================
  dnl  Specify optimization
  dnl ===================================================================
opt="-O3"
hiopt="-O6"
noopt="-O0"
AC_SUBST(opt)
AC_SUBST(hiopt)
AC_SUBST(noopt)
  dnl ===================================================================
  dnl  Writing config files
  dnl ===================================================================
AC_CONFIG_FILES(config/PConfig.h:config/PConfig.in)
AC_CONFIG_FILES(External/Makefile.patch:External/Makefile.patch.in)
AC_CONFIG_FILES(Utilities/setup/setup:Utilities/setup/setup.in)
AC_CONFIG_FILES(Utilities/setup/setupb:Utilities/setup/setupb.in)
AC_CONFIG_FILES(Utilities/setup/dol-user-env.sh:Utilities/setup/dol-user-env.sh.in)
AC_CONFIG_FILES(Utilities/setup/dol-user-env.csh:Utilities/setup/dol-user-env.csh.in)
AC_CONFIG_FILES(Utilities/setup/dol.pc:Utilities/setup/dol.pc.in)
AC_CONFIG_FILES(Utilities/procmgt/mkTrendDirs:Utilities/procmgt/mkTrendDirs.in)
AC_CONFIG_FILES(Utilities/procmgt/procmgt:Utilities/procmgt/procmgt.in)
AC_CONFIG_FILES(Utilities/procmgt/procsetup:Utilities/procmgt/procsetup.in)
])
